#!/usr/bin/perl

use JABBOT;

use utf8;
use Coro;
use AnyEvent;
use AnyEvent::XMPP::Util qw(prep_node_jid prep_bare_jid stringprep_jid);
use File::Slurp;
use IBH::TOPICS;
use JSON;
use RiveScript;
use Log::Any qw($log);
use Log::Any::Adapter;

use strict;
use warnings;

my $bot = 'marvin';
my $cfile = "conf/$bot/config";
my $cachedir = "cache/$bot";
my $usermap = "conf/usermap";

Log::Any::Adapter->set('Screen',
    min_level => 'debug',
    formatter => sub { (scalar localtime)." $_[1]"; },
);

$SIG{__WARN__} = sub {
    foreach my $msg (@_) {
	$log->warn($msg);
    }
};

# check values
die unless (-r $cfile);
die unless (-r $usermap);
die unless (-d $cachedir);


# load config
my %conf;
eval `cat "$cfile"`;
die $@ if($@);

my %users;
eval `cat "$usermap"`;


my %cv = (
    quit => AnyEvent->condvar,
    xmpp => AnyEvent->condvar,
    mqtt => AnyEvent->condvar,
);

my $jabo = new JABBOT(\%cv, $bot, \%conf);

foreach my $user (sort keys %users) {
    $jabo->rs->setSubstitution(
	$users{$user}->{initials} => $user,
	"$users{$user}->{sn} $users{$user}->{gn}" => $user,
	"$users{$user}->{gn} $users{$user}->{sn}" => $user,
	$users{$user}->{sn} => $user,
	$users{$user}->{gn} => $user,
    );
}

my %states;
my %counters;
my %presences;
my %lastjids;

my $w_status = AnyEvent->timer(
	interval => 28800, # 8h,
	cb => sub {
		$jabo->xmpp->set_presence('', $conf{status}->[rand scalar @{ $conf{status} }], 10);
	}
);

# Track presence updates
$jabo->xmpp->reg_cb(
    presence_update => sub {
	my ($client, $acc, $roster, $contact, $old_presence, $new_presence) = @_;

	my $jid = $contact->jid;
	my $user = prep_node_jid($jid);

	return if($user eq $bot);

	if(defined($new_presence)) {
	    $presences{$user}->{ stringprep_jid($new_presence->jid) } = {
		priority => $new_presence->priority,
		status => $new_presence->status,
		show => $new_presence->show || q(available),
	    };
	}
	elsif(defined($old_presence)) {
	    delete($presences{$user}->{ stringprep_jid($old_presence->jid) } );
	}

	if(defined($jid = get_jid($user))) {
	    my $nstate = get_state($jid);
	
	    $log->infof('presence %s => %s [%s]', $user, $jid, get_state($jid));
	}
    }
);

# DynNAV License Usage
$jabo->mqtt->subscribe(
    topic => qq($TOPIC_DYNNAV_SESSIONS/+),
    callback => sub {
	my ($topic, $json, $pub) = @_;

	return unless($json);

	my $data = decode_json($json);
	return unless(check_keys($topic, $data, qw(user count)));

	$counters{$data->{user}} = $data->{count};

	if($data->{count} > 1) {
	    my $jid = get_jid($data->{user});

	    return unless($jid);

	    async {
		$cv{xmpp}->recv;

		my $color = ($data->{count} > 2 ? q(red) : q(darkorange));

		$log->infof('DynNAV-Lic %s => %s', $jid, $data->{count});
		my $conn = $jabo->xmpp->find_account_for_dest_jid($jid)->connection;
		$conn->send_message($jid, q(chat), sub {},
		    body => qq(*ACHTUNG*: Du verwendest $data->{count} Navision-Lizenzen!),
		    html => qq(<strong style="color:$color">ACHTUNG: <em>Du verwendest $data->{count} Navision-Lizenzen!</em></strong>),
		);
	    }
	}
    },
);


# DynNAV Assigend Services
$jabo->mqtt->subscribe(
    topic => qq($TOPIC_DYNNAV_SERVICES/#),
    callback => sub {
	my ($topic, $json, $pub) = @_;

	return unless($json);

	my $data = decode_json($json);

	return unless(check_keys($topic, $data, qw(Document_No Planned_Res_No Current_Document_Status DisplayName DisplayTaskDesc)));

	if($data->{Current_Document_Status} =~ /^(CL|DL)-NEU$/) {
	    $log->infof('DynNAV-Srv %s: ignoring state %s', $data->{Document_No}, $data->{Current_Document_Status});
	    return;
	}

	my $jid = get_jid($data->{Planned_Res_No});
	unless($jid) {
	    $log->infof('DynNAV-Srv %s: jid for %s is unknown', $data->{Document_No}, $data->{Planned_Res_No});
	    return;
	}

	async {
	    $cv{xmpp}->recv;

	    my $bgcolor = q(yellow);
	    if($data->{Current_Document_Status} =~ /problem/i) {
		$bgcolor = q(orange);
	    }
	    elsif($data->{Current_Document_Status} =~ /erled|abgeschl|freig/i) {
		$bgcolor = q(lightgreen);
	    }
	    elsif($data->{Current_Document_Status} =~ /-w-/i) {
		$bgcolor = q(lightskyblue);
	    }
	    elsif($data->{Current_Document_Status} =~ /zuweis/i) {
		$bgcolor = q(mistyrose);
	    }

	    $log->infof('DynNAV-Srv %s => %s', $jid, $data->{Document_No});
	    my $url = $data->{Document_No};
	    if($data->{Link} && !($jid =~ /voip.lan.ibh.de/)) {
		my $link = $data->{Link};
		$link =~ s/&/&amp;/g;
		$url = "<a href=\"$link\">$url</a>";
	    }
	    my ($comp) = $jabo->xmpp_escape($data->{DisplayName});
	    my ($task) = $jabo->xmpp_escape($data->{DisplayTaskDesc});
	    my $conn = $jabo->xmpp->find_account_for_dest_jid($jid)->connection;
	    $conn->send_message($jid, q(chat), sub {},
		body => qq($data->{Document_No} - $data->{Current_Document_Status}\n$comp: $task),
		html => qq(<strong>$url &#8594; <span style="background-color:$bgcolor">$data->{Current_Document_Status}</span></strong><br/>\n<strong>$comp:</strong> <em>$task</em>),
	    );
	}
    },
);


$jabo->start;

$cv{quit}->recv;

sub check_keys {
    my ($topic, $href, @keys) = @_;

    my @found = grep { (exists($href->{$_}) ? 1 : 0) } @keys;
    my $valid = scalar @keys == scalar @found;

    $log->warnf("Missing keys in topic '%s'", $topic) unless($valid);

    return $valid;
}

sub map_show {
    my $show = shift;

    my %map = (
	chat => 128,
	available => 64,
	away => 32,
	dnd => 16,
	xa => 8,
    );

    return $map{$show} if(exists($map{$show}));

    return 8;
}

sub get_state {
    my $jid = shift;
    my $user = prep_node_jid($jid);

    return q(offline) unless(exists($presences{$user}));

    my ($show) = sort {
	    return map_show( $presences{$user}->{$b}->{show} ) <=> map_show( $presences{$user}->{$a}->{show} );
    } grep {
	prep_bare_jid($_) eq $jid;
    } keys %{ $presences{$user} };

    return (defined($show) ? $presences{$user}->{$show}->{show} : 'offline');
}

sub get_jid {
    my $user = lc(shift);

    if(exists($presences{$user})) {
	my ($jid) = sort {
	    my $c = map_show( $presences{$user}->{$b}->{show} ) <=> map_show( $presences{$user}->{$a}->{show} );

	    return $c unless($c == 0);

	    return $presences{$user}->{$b}->{priority} <=> $presences{$user}->{$a}->{priority};
	} grep {
	    (($presences{$user}->{$_}->{priority} || 0) > 0);
	} keys %{ $presences{$user} };

	if($jid) {
	    $lastjids{$user} = $jid;
	    return prep_bare_jid($jid) if($jid);
	}
    }

    return $lastjids{$user} if(exists($lastjids{$user}));

    return "$user\@voip.lan.ibh.de";
}
